/*
 * GEOServerGPSClient.java
 *
 * Created on 1  2007 ., 14:21
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package com.substanceofcode.bluetooth;

import com.substanceofcode.tracker.controller.Controller;
import javax.microedition.midlet.MIDlet;
import java.io.*;
import java.util.Calendar;
import java.util.Date;
import javax.microedition.io.*;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
import com.substanceofcode.tracker.model.Settings;
import com.substanceofcode.bluetooth.GEOServerGPSClientCFGForm;
import com.substanceofcode.bluetooth.GEOServerGPSClientStatusForm;
import javax.microedition.lcdui.Alert;

/**
 *
 * @author PAV
 */
public class GEOServerGPSClient implements Runnable 
{
    private static final String CFG_ServerURL = "GEOSERVERGPSCLIENTCFG_ServerURL";
    private static final String CFG__GPSFixThreshold_MaxDistance = "GEOSERVERGPSCLIENTCFG__GPSFixThreshold_MaxDistance";
    private static final String CFG__GPSFixThreshold_UpdateInterval = "GEOSERVERGPSCLIENTCFG__GPSFixThreshold_UpdateInterval";
    private static final String CFG_ObjectID = "GEOSERVERGPSCLIENTCFG_ObjectID";
    private static final String CFG_ObjectUserPassword = "GEOSERVERGPSCLIENTCFG_ObjectUserPassword";
    //.
    //.
    public Controller controller;
    //.
    public String ServerURL = "";
    public String ServerErrorString = "";
    //.
    private int GPSFix_ProcessedCount = 0; 
    private Date GPSFix_Time = Calendar.getInstance().getTime();
    private double GPSFix_Latitude;
    private double GPSFix_Longitude;
    private double GPSFix_Precision;
    //.
    public int GPSFixThreshold_MaxDistance = 100;
    public int GPSFixThreshold_UpdateInterval = 30;
    //.
    public int ObjectID = 0;
    public String ObjectUserPassword = "";
    //.
    public int CalibrationGeodesyPointID = 0;
    public String CalibrationUserPassword = "";
    //.
    private Thread thread;
    //.
    private GEOServerGPSClientCFGForm CFGForm = null;
    private GEOServerCalibrationForm CalibrationForm = null;
    private GEOServerGPSClientStatusForm StatusForm = null;
   
    /** Creates a new instance of GEOServerGPSClient */
    public GEOServerGPSClient(Controller pcontroller) 
    {
        controller = pcontroller;
        //.
        LoadConfiguration();
        //.
        thread = new Thread(this);
        thread.start();
    }
    
    public void LoadConfiguration()
    {
        ServerURL = "";
        GPSFixThreshold_MaxDistance = 100;
        try {
            Settings m_settings = Settings.getInstance(controller.m_midlet);
            ServerURL = m_settings.getStringProperty(CFG_ServerURL, "");
            GPSFixThreshold_MaxDistance = m_settings.getIntProperty(CFG__GPSFixThreshold_MaxDistance,100);
            GPSFixThreshold_UpdateInterval = m_settings.getIntProperty(CFG__GPSFixThreshold_UpdateInterval,30);
            ObjectID = m_settings.getIntProperty(CFG_ObjectID, 0);
            ObjectUserPassword = m_settings.getStringProperty(CFG_ObjectUserPassword, "");
        }
        catch(Exception ex) 
        {
            System.err.println("Error occured while creating an instance of Settings class: " + ex.toString());
        }
    }
    
    public void SaveConfiguration()
    {
        try {
            Settings m_settings = Settings.getInstance(controller.m_midlet);
            m_settings.setStringProperty(CFG_ServerURL,ServerURL);
            m_settings.setIntProperty(CFG__GPSFixThreshold_MaxDistance,GPSFixThreshold_MaxDistance);
            m_settings.setIntProperty(CFG__GPSFixThreshold_UpdateInterval,GPSFixThreshold_UpdateInterval);
            m_settings.setIntProperty(CFG_ObjectID,ObjectID);
            m_settings.setStringProperty(CFG_ObjectUserPassword,ObjectUserPassword);
            m_settings.save(true);
        }
        catch(Exception ex) 
        {
            System.err.println("Error occured while creating an instance of Settings class: " + ex.toString());
        }
    }
    
    public void ShowCFGForm()
    {
        if (CFGForm == null) 
            CFGForm = new GEOServerGPSClientCFGForm(this);
        controller.m_display.setCurrent(CFGForm);
    }
    
    public GEOServerCalibrationForm GetCalibrationForm()
    {
        if (CalibrationForm == null) CalibrationForm = new GEOServerCalibrationForm(this);
        return CalibrationForm;
    }
    
    public GEOServerGPSClientStatusForm GetStatusForm()
    {
        if (StatusForm == null) StatusForm = new GEOServerGPSClientStatusForm(this);
        return StatusForm;
    }
    
    public void PromptNetworkAccess() throws IOException
    {
        try
        {
            SocketConnection sc = null;
            DataOutputStream os = null;
            try
            {
                // a socket connection is established with the remote server.
                sc = (SocketConnection) Connector.open("socket://"+ServerURL);
                sc.setSocketOption(SocketConnection.DELAY, 1000/*seconds*/);
                sc.setSocketOption(SocketConnection.KEEPALIVE,1);
                sc.setSocketOption(SocketConnection.SNDBUF, 10000);
                os = sc.openDataOutputStream();
                //. prepare message 
                short ServiceID = 0; //. null service as test connection
                byte[] MessageArray = new byte[2];
                byte[] BA = ConvertInt16ToLEByteArray(ServiceID);
                //. send data top the server
                os.write(MessageArray);
                os.flush();
            } 
            finally 
            {
                // free up the I/O stream after use
                /*try { if (os != null ) os.close(); }
                catch (IOException ignored) {}*/
  
                // free up the socket connection after use
                try { if ( sc != null ) sc.close(); }
                catch (IOException ignored) {}
            }
        }
        catch (Exception E)
        {
            ServerErrorString = E.getMessage();
        };
    }
    
    public double GetDistance(double StartLat, double StartLong,  double EndLat, double EndLong)
    /* COPYRIGHT DelphiWorld */
    {
        double fPhimean;
        double fdLambda;
        double fdPhi; 
        double fAlpha;
        double fRho;
        double fNu;
        double fR;
        double fz;
        double fTemp;
        double Distance;
        double Bearing;
        //. consts
        double D2R = 0.017453;
        double R2D = 57.295781; 
        double a = 6378137.0;
        double b = 6356752.314245; 
        double e2 = 0.006739496742337; 
        double f = 0.003352810664747; 
        
        if (Double.isNaN(StartLat) || Double.isNaN(StartLong) || Double.isNaN(EndLat) || Double.isNaN(EndLong))
                return Double.NaN; //. ->
        
        //           
        fdLambda = (StartLong - EndLong) * D2R;
        fdPhi = (StartLat - EndLat) * D2R;
        fPhimean = ((StartLat + EndLat) / 2.0) * D2R;
        //        
        fTemp = 1 - e2 * (Float11.pow(Math.sin(fPhimean), 2));
        fRho = (a * (1 - e2)) / Float11.pow(fTemp, 1.5);
        fNu = a / (Math.sqrt(1 - e2 * (Math.sin(fPhimean) * Math.sin(fPhimean))));
        //   
        fz = Math.sqrt(Float11.pow(Math.sin(fdPhi / 2.0), 2) + Math.cos(EndLat * D2R) * Math.cos(StartLat * D2R) * Float11.pow(Math.sin(fdLambda / 2.0), 2));
        fz = 2 * Float11.asin(fz);
        //  
        fAlpha = Math.cos(EndLat * D2R) * Math.sin(fdLambda) * 1 / Math.sin(fz);
        fAlpha = Float11.asin(fAlpha);
        //   
        fR = (fRho * fNu) / ((fRho * Float11.pow(Math.sin(fAlpha), 2)) + (fNu * Float11.pow(Math.cos(fAlpha), 2)));
        //    
        Distance = (fz * fR);
        if ((StartLat < EndLat) && (StartLong < EndLong)) 
            Bearing = Math.abs(fAlpha * R2D);
        else if ((StartLat < EndLat) && (StartLong > EndLong)) 
            Bearing = 360 - Math.abs(fAlpha * R2D);
        else if ((StartLat > EndLat) && (StartLong > EndLong)) 
            Bearing = 180 + Math.abs(fAlpha * R2D);
        else if ((StartLat > EndLat) && (StartLong < EndLong)) 
            Bearing = 180 - Math.abs(fAlpha * R2D);
        //.
        return Distance;
    }
    
    private boolean _AcceptNewPosition(Date pGPSFix_Time, double pGPSFix_Latitude, double pGPSFix_Longitude, double pGPSFix_Precision, GpsPosition Pos)
    {
        long secondsSinceLastPosition = (Pos.getDate().getTime() - pGPSFix_Time.getTime())/1000;
        double Distance = GetDistance(pGPSFix_Latitude,pGPSFix_Longitude, Pos.getLatitude(),Pos.getLongitude());
        return ((GPSFix_ProcessedCount == 0) || 
                ((secondsSinceLastPosition > GPSFixThreshold_UpdateInterval) && (Distance > pGPSFix_Precision)) ||
                ((GPSFixThreshold_MaxDistance > 0) && (Distance >= GPSFixThreshold_MaxDistance)) 
               );
    }
    
    public boolean AcceptNewPosition(GpsPosition GPSFixPos, GpsPosition Pos)
    {
        return _AcceptNewPosition(GPSFixPos.getDate(),GPSFixPos.getLatitude(),GPSFixPos.getLongitude(),Pos.getPrecision(), Pos);
    }
    
    public boolean AcceptNewPosition(GpsPosition Pos)
    {
        return _AcceptNewPosition(GPSFix_Time,GPSFix_Latitude,GPSFix_Longitude,Pos.getPrecision(), Pos);
    }
    
    public synchronized void SetGPSFixData(GpsPosition Pos)
    {
        if (AcceptNewPosition(Pos))
        {
            GPSFix_Time = Pos.getDate();
            GPSFix_Latitude = Pos.getLatitude();
            GPSFix_Longitude = Pos.getLongitude();            
            GPSFix_Precision = Pos.getPrecision();            
            GPSFix_ProcessedCount++;
        }
    }
    
    private synchronized GPSFixData GetGPSFixData()
    {
            GPSFixData Data = new GPSFixData();
            Data.Latitude = GPSFix_Latitude;
            Data.Longitude = GPSFix_Longitude;            
            Data.Precision = GPSFix_Precision;            
            Data.ID = GPSFix_ProcessedCount;
            return Data;
    }
    
    public byte[] ConvertInt16ToLEByteArray(short V) throws IOException
    {
        byte E;
        //.
        ByteArrayOutputStream bos = new java.io.ByteArrayOutputStream(8);
        DataOutputStream dos = new DataOutputStream(bos);
        dos.writeShort(V);
        //.
        byte[] R = bos.toByteArray();
        //.
        E = R[0]; R[0] = R[1]; R[1] = E;
        //.
        return R;
    }
    
    public byte[] ConvertInt32ToLEByteArray(int V) throws IOException
    {
        byte E;
        //.
        ByteArrayOutputStream bos = new java.io.ByteArrayOutputStream(8);
        DataOutputStream dos = new DataOutputStream(bos);
        dos.writeInt(V);
        //.
        byte[] R = bos.toByteArray();
        //.
        E = R[0]; R[0] = R[1]; R[1] = E;
        E = R[2]; R[2] = R[3]; R[3] = E;
        //.
        E = R[0]; R[0] = R[2]; R[2] = E; E = R[1]; R[1] = R[3]; R[3] = E;
        //.
        return R;
    }
    
    public byte[] ConvertDoubleToLEByteArray(double V) throws IOException
    {
        byte E;
        //.
        ByteArrayOutputStream bos = new java.io.ByteArrayOutputStream(8);
        DataOutputStream dos = new DataOutputStream(bos);
        dos.writeDouble(V);
        //.
        byte[] R = bos.toByteArray();
        //.
        E = R[0]; R[0] = R[1]; R[1] = E;
        E = R[2]; R[2] = R[3]; R[3] = E;
        E = R[4]; R[4] = R[5]; R[5] = E;
        E = R[6]; R[6] = R[7]; R[7] = E;
        //.
        E = R[0]; R[0] = R[2]; R[2] = E; E = R[1]; R[1] = R[3]; R[3] = E;
        E = R[4]; R[4] = R[6]; R[6] = E; E = R[5]; R[5] = R[7]; R[7] = E;
        //.
        E = R[0]; R[0] = R[4]; R[4] = E; E = R[1]; R[1] = R[5]; R[5] = E; E = R[2]; R[2] = R[6]; R[6] = E; E = R[3]; R[3] = R[7]; R[7] = E;
        //.
        return R;
    }
    
    public void SendGPSData(GPSFixData Data) throws IOException
    {
            SocketConnection sc = null;
            DataOutputStream os = null;
            try
            {
                // a socket connection is established with the remote server.
                sc = (SocketConnection) Connector.open("socket://"+ServerURL);
                sc.setSocketOption(SocketConnection.DELAY, 1000/*seconds*/);
                sc.setSocketOption(SocketConnection.KEEPALIVE,1);
                sc.setSocketOption(SocketConnection.SNDBUF, 10000);
                os = sc.openDataOutputStream();
                //. prepare message (ServiceID #2)
                short ServiceID = 2;
                byte[] MessageArray = new byte[2+4/*Preamble*/+3*8+4/*BodyCRC*/];
                int Idx = 0;
                //. prepare ServiceID
                byte[] BA = ConvertInt16ToLEByteArray(ServiceID);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                //. prepare ObjectID
                BA = ConvertInt32ToLEByteArray(ObjectID);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                //. prepare Latitude,Longitude,Precision
                int BodyStart = Idx;
                BA = ConvertDoubleToLEByteArray(Data.Latitude);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                BA = ConvertDoubleToLEByteArray(Data.Longitude);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                BA = ConvertDoubleToLEByteArray(Data.Precision);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                //. store BodyCRC
                int CRC = 0;
                int V;
                int I = 0;
                while (I < (3*8)) 
                {
                    V = (MessageArray[BodyStart+I] & 0x000000FF)+((MessageArray[BodyStart+I+1] & 0x000000FF) << 8)+((MessageArray[BodyStart+I+2] & 0x000000FF) << 16)+((MessageArray[BodyStart+I+3] & 0x000000FF) << 24);
                    CRC = (CRC^V);
                    //.
                    I+=4;
                };
                BA = ConvertInt32ToLEByteArray(CRC);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length);  
                //. encrypting message body by user-password
                byte[] ObjectUserPasswordArray = ObjectUserPassword.getBytes("windows-1251");
                int ObjectUserPasswordIdx = 0;
                for (I = (2+4) /*skip preamble*/; I < MessageArray.length; I++)
                {
                    MessageArray[I] = (byte)(MessageArray[I]+ObjectUserPasswordArray[ObjectUserPasswordIdx]);
                    //.
                    ObjectUserPasswordIdx++;
                    if (ObjectUserPasswordIdx >= ObjectUserPasswordArray.length) ObjectUserPasswordIdx = 0;
                }
                //. send data top the server
                os.write(MessageArray);
                os.flush();
            } 
            finally 
            {
                // free up the I/O stream after use
                /*try { if (os != null ) os.close(); }
                catch (IOException ignored) {}*/
  
                // free up the socket connection after use
                try { if ( sc != null ) sc.close(); }
                catch (IOException ignored) {}
            }
    }
    
    public void SendStatusEvent(GPSFixData Data, int StatusEventID) throws IOException
    {
            SocketConnection sc = null;
            DataOutputStream os = null;
            try
            {
                // a socket connection is established with the remote server.
                sc = (SocketConnection) Connector.open("socket://"+ServerURL);
                sc.setSocketOption(SocketConnection.DELAY, 1000/*seconds*/);
                sc.setSocketOption(SocketConnection.KEEPALIVE,1);
                sc.setSocketOption(SocketConnection.SNDBUF, 10000);
                os = sc.openDataOutputStream();
                //. prepare message (ServiceID #4)
                short ServiceID = 4;
                byte[] MessageArray = new byte[2+4/*Preamble*/+4/*StatusEventID*/+3*8+4/*BodyCRC*/];
                int Idx = 0;
                //. prepare ServiceID
                byte[] BA = ConvertInt16ToLEByteArray(ServiceID);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                //. prepare ObjectID
                BA = ConvertInt32ToLEByteArray(ObjectID);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                int BodyStart = Idx;
                //. prepare StatusEventID
                BA = ConvertInt32ToLEByteArray(StatusEventID);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                //. prepare Latitude,Longitude,Precision
                BA = ConvertDoubleToLEByteArray(Data.Latitude);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                BA = ConvertDoubleToLEByteArray(Data.Longitude);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                BA = ConvertDoubleToLEByteArray(Data.Precision);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                //. store BodyCRC
                int CRC = 0;
                int V;
                int I = 0;
                while (I < (4+3*8)) 
                {
                    V = (MessageArray[BodyStart+I] & 0x000000FF)+((MessageArray[BodyStart+I+1] & 0x000000FF) << 8)+((MessageArray[BodyStart+I+2] & 0x000000FF) << 16)+((MessageArray[BodyStart+I+3] & 0x000000FF) << 24);
                    CRC = (CRC^V);
                    //.
                    I+=4;
                };
                BA = ConvertInt32ToLEByteArray(CRC);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length);  
                //. encrypting message body by user-password
                byte[] ObjectUserPasswordArray = ObjectUserPassword.getBytes("windows-1251");
                int ObjectUserPasswordIdx = 0;
                for (I = (2+4) /*skip preamble*/; I < MessageArray.length; I++)
                {
                    MessageArray[I] = (byte)(MessageArray[I]+ObjectUserPasswordArray[ObjectUserPasswordIdx]);
                    //.
                    ObjectUserPasswordIdx++;
                    if (ObjectUserPasswordIdx >= ObjectUserPasswordArray.length) ObjectUserPasswordIdx = 0;
                }
                //. send data top the server
                os.write(MessageArray);
                os.flush();
            } 
            finally 
            {
                // free up the I/O stream after use
                /*try { if (os != null ) os.close(); }
                catch (IOException ignored) {}*/
  
                // free up the socket connection after use
                try { if ( sc != null ) sc.close(); }
                catch (IOException ignored) {}
            }
    }
    
    public void CalibrateGeodesyPoint(GPSFixData Data) throws IOException
    {
            SocketConnection sc = null;
            DataOutputStream os = null;
            try
            {
                // a socket connection is established with the remote server.
                sc = (SocketConnection) Connector.open("socket://"+ServerURL);
                sc.setSocketOption(SocketConnection.DELAY, 1000/*seconds*/);
                sc.setSocketOption(SocketConnection.KEEPALIVE,1);
                sc.setSocketOption(SocketConnection.SNDBUF, 10000);
                os = sc.openDataOutputStream();
                //. prepare message (ServiceID #3)
                short ServiceID = 3;
                byte[] MessageArray = new byte[2+4/*Preamble*/+3*8+4/*BodyCRC*/];
                int Idx = 0;
                //. prepare ServiceID
                byte[] BA = ConvertInt16ToLEByteArray(ServiceID);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                //. prepare GeodesyPointID
                BA = ConvertInt32ToLEByteArray(CalibrationGeodesyPointID);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                //. prepare Latitude,Longitude,Precision
                int BodyStart = Idx;
                BA = ConvertDoubleToLEByteArray(Data.Latitude);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                BA = ConvertDoubleToLEByteArray(Data.Longitude);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                BA = ConvertDoubleToLEByteArray(Data.Precision);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length); Idx+=BA.length; 
                //. store BodyCRC
                int CRC = 0;
                int V;
                int I = 0;
                while (I < (3*8)) 
                {
                    V = (MessageArray[BodyStart+I] & 0x000000FF)+((MessageArray[BodyStart+I+1] & 0x000000FF) << 8)+((MessageArray[BodyStart+I+2] & 0x000000FF) << 16)+((MessageArray[BodyStart+I+3] & 0x000000FF) << 24);
                    CRC = (CRC^V);
                    //.
                    I+=4;
                };
                BA = ConvertInt32ToLEByteArray(CRC);
                System.arraycopy(BA,0,MessageArray,Idx,BA.length);  
                //. encrypting message body by user-password
                byte[] CalibrationUserPasswordArray = CalibrationUserPassword.getBytes("windows-1251");
                int CalibrationUserPasswordIdx = 0;
                for (I = (2+4) /*skip preamble*/; I < MessageArray.length; I++)
                {
                    MessageArray[I] = (byte)(MessageArray[I]+CalibrationUserPasswordArray[CalibrationUserPasswordIdx]);
                    //.
                    CalibrationUserPasswordIdx++;
                    if (CalibrationUserPasswordIdx >= CalibrationUserPasswordArray.length) CalibrationUserPasswordIdx = 0;
                }
                //. send data top the server
                os.write(MessageArray);
                os.flush();
            } 
            finally 
            {
                // free up the I/O stream after use
                /*try { if (os != null ) os.close(); }
                catch (IOException ignored) {}*/
  
                // free up the socket connection after use
                try { if ( sc != null ) sc.close(); }
                catch (IOException ignored) {}
            }
    }
    
    public void run() 
    {
        int GPSFixID = 0;
        GPSFixData FixData;
        while (Thread.currentThread() == thread) 
        {
            try 
            {
                FixData = GetGPSFixData();
                if (FixData.ID != GPSFixID)
                {
                    //. send gps fix data to the remote server
                    try
                    {
                        SendGPSData(FixData);
                        ServerErrorString = "";
                    }
                    catch (Exception E)
                    {
                        ServerErrorString = E.getMessage();
                    };
                    //.
                    GPSFixID = FixData.ID;
                }
                else
                    Thread.sleep(1000);
            }
            catch (Exception E) 
            {}  
        }        
    }

}

class GPSFixData
{
    public double Latitude;
    public double Longitude;
    public double Precision;
    public int ID;
}

/**
 * <p>Title: Class for float-point calculations in J2ME applications CLDC 1.1</p>
 * <p>Description: Useful methods for float-point calculations which absent in native Math class</p>
 * <p>Copyright: Copyright (c) 2004 Nick Henson</p>
 * <p>Company: UNTEH</p>
 * <p>License: Free use only for non-commercial purpose</p>
 * <p>If you want to use all or part of this class for commercial applications then take into account these conditions:</p>
 * <p>1. I need a one copy of your product which includes my class with license key and so on</p>
 * <p>2. Please append my copyright information henson.midp.Float (C) by Nikolay Klimchuk on About screen of your product</p>
 * <p>3. If you have web site please append link <a href=http://henson.newmail.ru>Nikolay Klimchuk</a> on the page with description of your product</p>
 * <p>That's all, thank you!</p>
 * @author Nikolay Klimchuk http://henson.newmail.ru
 * @version 0.5
 */

class Float11
{
  /** Square root from 3 */
  final static public double SQRT3 = 1.732050807568877294;
  /** Log10 constant */
  final static public double LOG10 = 2.302585092994045684;
  /** ln(0.5) constant */
  final static public double LOGdiv2 = -0.6931471805599453094;
  //
  static public double acos(double x)
  {
    double f=asin(x);
    if(f==Double.NaN)
      return f;
    return Math.PI/2-f;
  }

  static public double asin(double x)
  {
    if( x<-1. || x>1. ) return Double.NaN;
    if( x==-1. ) return -Math.PI/2;
    if( x==1 ) return Math.PI/2;
    return atan(x/Math.sqrt(1-x*x));
  }

  static public double atan(double x)
  {
      boolean signChange=false;
      boolean Invert=false;
      int sp=0;
      double x2, a;
      // check up the sign change
      if(x<0.)
      {
          x=-x;
          signChange=true;
      }
      // check up the invertation
      if(x>1.)
      {
          x=1/x;
          Invert=true;
      }
      // process shrinking the domain until x<PI/12
      while(x>Math.PI/12)
      {
          sp++;
          a=x+SQRT3;
          a=1/a;
          x=x*SQRT3;
          x=x-1;
          x=x*a;
      }
      // calculation core
      x2=x*x;
      a=x2+1.4087812;
      a=0.55913709/a;
      a=a+0.60310579;
      a=a-(x2*0.05160454);
      a=a*x;
      // process until sp=0
      while(sp>0)
      {
          a=a+Math.PI/6;
          sp--;
      }
      // invertation took place
      if(Invert) a=Math.PI/2-a;
      // sign change took place
      if(signChange) a=-a;
      //
      return a;
  }

  static public double atan2(double y, double x)
  {
    // if x=y=0
    if(y==0. && x==0.)
      return 0.;
    // if x>0 atan(y/x)
    if(x>0.)
      return atan(y/x);
    // if x<0 sign(y)*(pi - atan(|y/x|))
    if(x<0.)
    {
      if(y<0.)
        return -(Math.PI-atan(y/x));
      else
        return Math.PI-atan(-y/x);
    }
    // if x=0 y!=0 sign(y)*pi/2
    if(y<0.)
      return -Math.PI/2.;
    else
      return Math.PI/2.;
  }

  static public double exp(double x)
  {
    if(x==0.)
      return 1.;
    //
    double f=1;
    long d=1;
    double k;
    boolean isless=(x<0.);
    if(isless)
      x=-x;
    k=x/d;
    //
    for(long i=2; i<50; i++)
    {
      f=f+k;
      k=k*x/i;
    }
    //
    if(isless)
      return 1/f;
    else
      return f;
  }

  static private double _log(double x)
  {
    if(!(x>0.))
      return Double.NaN;
    //
    double f=0.0;
    //
    int appendix=0;
    while(x>0.0 && x<=1.0)
    {
      x*=2.0;
      appendix++;
    }
    //
    x/=2.0;
    appendix--;
    //
    double y1=x-1.;
    double y2=x+1.;
    double y=y1/y2;
    //
    double k=y;
    y2=k*y;
    //
    for(long i=1; i<50; i+=2)
    {
      f+=k/i;
      k*=y2;
    }
    //
    f*=2.0;
    for(int i=0; i<appendix; i++)
      f+=LOGdiv2;
    //
    return f;
  }

  static public double log(double x)
   {
     if(!(x>0.))
       return Double.NaN;
     //
     if(x==1.0)
       return 0.0;
     // Argument of _log must be (0; 1]
     if (x>1.)
     {
       x=1/x;
       return -_log(x);
     };
     //
     return _log(x);
   }

   static public double log10(double x)
   {
    return log(x)/LOG10;
   }

/*
  static public double log10(double x)
  {
    if(!(x>0.))
      return Double.NaN;
    //
    boolean neg=false;
    if(x<0)
    {
      neg=true;
      x=-x;
    }
    //
    int index=0;
    if(x>1.)
    {
      // Great 1
      while(x>1.)
      {
        x=x/10;
        index++;
      }
    }
    else
    {
      // Less 1
      while(x<1.)
      {
        x=x*10.;
        index--;
      }
    }
    //
    double res=index;
    if(x!=1.)
      res=res+(log(x)/LOG10);
    //
    if(neg)
      return 1./res;
    else
      return res;
  }
*/
  static public double pow(double x, double y)
  {
    if(x==0.)
      return 0.;
    if(x==1.)
      return 1.;
    if(y==0.)
      return 1.;
    if(y==1.)
      return x;
    //
    long l=(long)Math.floor(y);
    boolean integerValue=(y==(double)l);
    //
    if(integerValue)
    {
      boolean neg=false;
      if(y<0.)
        neg=true;
      //
      double result=x;
      for(long i=1; i<(neg?-l:l); i++)
        result=result*x;
      //
      if(neg)
        return 1./result;
      else
        return result;
    }
    else
    {
      if(x>0.)
        return exp(y*log(x));
      else
        return Double.NaN;
    }
  }
}
